/*
 *  ImageProcessing.cpp
 *  Assignment8
 *
 *  Created by Zach Gildersleeve on 3/19/07.
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 *
 */

#include "ImageProcessing.h"
#include "Filter.h"
#include "Sampler.h"

int writeSwitch = 1;		// switch to control writing image
int doOnce = 1;				// do something once
int mode = 1;				// switch for main rendering window

int xRes = 512;				// image X resolution
int yRes = 512;				// image Y resolution
Image *image;	// the image to be written
int mainWindow;				// pointer to the main window

int currentStep = (yRes / 4);		// value to incriment with during progressive rendering

double t1, t2, t3, t4, r1, r2;		// values for time counters
double t5 = 0;						// counter for progressive rendering

char filename[] = "function.ppm";
Scene *scene;

int superCounter = 0;

/**
*	Simple glut idle function.
 */
void myGlutIdle(void)
{
	glutPostRedisplay();
}

/**
*	Simple glut reshape function.
 *
 *	@param	x	the new x window coord
 *	@param	y	the new y window coord
 */
void myGlutReshape(int	x, int y)
{
	glViewport(0, 0, xRes, yRes);
}

/**
*	GLUT keyboard control.  Pressing 'S' at any point
 *	will overwrite and save the current render.
 *
 *	@param	key	the key pressed
 *	@param	x	?
 *	@param	y	?
 */
void myKeyboard(unsigned char key, int x, int y)
{
	switch(key)
	{
		case 's':	// save image
		case 'S':
			image->write(filename);
			break;
		case 'P':
		case 'p':
			if (mode != 1)
				mode = 1;
			else
				mode = 0;
			break;
		case 27:	// ESC quits program
			exit(0);
			break;
		default:
			break;
	}
}

void mySpecialKeys(int key, int x, int y)
{
	switch(key)
	{
		case 's':	// save image
		case 'S':
			image->write(filename);
			break;
		case 27:	// ESC quits program
			exit(0);
			break;
		default:
			break;
	}
}

/**
*	Display function.  Draws the image pixels to the OpenGL window depending
 *	on how much of the image is raytraced, or if currentStep is anything but 1,
 *	which will progressively render the image.
 */
void myDisplay(void)
{
	glClear(GL_COLOR_BUFFER_BIT);
	glLoadIdentity();
	glColor3f(1.0,1.0,1.0);
	glLoadIdentity(); 
	glOrtho(-xRes/2, xRes/2, -yRes/2, yRes/2, 0, 2); 

	glDrawPixels(xRes, yRes, GL_RGB, GL_UNSIGNED_BYTE, image->imageData[0]);
	glutSwapBuffers();		
}

double function(double x, double y)
{
	return 0.5 * (cos(500 * M_PI * (x * x + y * y)) + 1.0);
}



/**
*	Main program, allows for two rendering options.  Option 1
 *	renders the raytraces image completely, writes it to a file,
 *	and then displays the result in an OpenGL window.  Option 2
 *	raytraces and displays the image at the same time, rendering in
 *	a progressive fashion.  Also contains the glutInit* and glut*Func
 *	calls.
 */
int main (int argc, char** argv)
{	
	t1 = Timer::currentSeconds();
	cout << "Pre-Processing...\n";
	
	image = new Image(xRes, yRes);
	
	t2 = Timer::currentSeconds();
	cout << "Starting Render...\n";

	Color result;
	
	double xMult = (xRes / 2.0) + 0.5;
	double yMult = (yRes / 2.0) + 0.5;
	double xInv = 2.0 / xRes;
	double yInv = 2.0 / yRes;
	double temp;
	
	for(int i=0;i<yRes;i++){
		
		if (i % 16 == 0)
			std::cout << "\tRendering Line" << i << "\n";
		
		double y = (i - yMult) * yInv; 
		
		for(int j=0;j<xRes;j++){ 
			
			double x = (j - xMult) * xInv;
			
			double *weight;
			double *xs, *ys;
			int subDivisions = 3;
			int support = 3;
			int numberSamples = support * subDivisions;	
			int numberSamplesSquared = numberSamples * numberSamples;
			
			weight = new double[numberSamplesSquared];
			xs = new double[numberSamplesSquared];
			ys = new double[numberSamplesSquared];
			
//			BoxFilter filter = BoxFilter(1);
//			TriangleFilter filter = TriangleFilter(support);
			GaussianFilter filter = GaussianFilter(support);
			
			
//			UniformSampler sampler = UniformSampler(subDivisions);
			JitteredSampler sampler = JitteredSampler(subDivisions);
			sampler.getSamples(x, y, support, xs, ys);
			
			filter.getWeights(weight, numberSamplesSquared, xs, ys);

			int weightCounter;
			double totalWeight = 0.0;
			double sum = 0;
			temp = 0;
			
			for (weightCounter = 0; weightCounter < numberSamplesSquared; weightCounter++)
			{
				totalWeight += weight[weightCounter];

				temp += function(x + xs[weightCounter] * xInv, y + ys[weightCounter] * yInv) * weight[weightCounter];
				
			//	printf("thisWeight = %f totalWeight = %f ", weight[weightCounter], totalWeight);
			//	sum = weight[weightCounter] * temp;
			}
			//printf("\ntotalWeight = %f\n", totalWeight, temp, temp / totalWeight);
			temp /= totalWeight;
			//temp = sum / totalWeight;
			image->set(j, i, Color(temp, temp, temp));
			
			delete weight;
			delete xs;
			delete ys;
		} 
	} 
		
		cout << "Done\n\nWriting File\n";
		t3 = Timer::currentSeconds();
		image->write(filename);
		
		//writeSwitch = 0;
		t4 = Timer::currentSeconds();
		
		cout << "\n--------------------\nSetup time:\t\t" << setprecision(3) << fixed << t2 - t1 << "\n";
		cout << "Render time:\t\t" << t3 - t2 << "\n";
		cout << "Image write time:\t" << t4 - t3 << "\n";
		cout << "Total Processing:\t" << t4 - t1 << "\ntri intersect called " << superCounter << " times\n--------------------\n";
	
	cout << "Displaying Image...\n";
	
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_SINGLE | GLUT_RGB);
	glutInitWindowSize(xRes, yRes);
	glutInitWindowPosition(100, 100);
	
	mainWindow = glutCreateWindow("Raytracer");
	glewInit();
	
	glClearColor(0.0, 0.0, 0.0, 0);
	
	glutIdleFunc(myGlutIdle);
	glutDisplayFunc(myDisplay);
	glutReshapeFunc(myGlutReshape);
	glutKeyboardFunc(myKeyboard);
	glutSpecialFunc(mySpecialKeys);
	
	glutMainLoop();
	
    return 0;
}