/*
 *  PhongImageMaterial.cpp
 *  Assignment7
 *
 *  Created by Zach Gildersleeve on 3/12/07.
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 *
 */

#include "PhongImageMaterial.h"

PhongImageMaterial::PhongImageMaterial(const char* newFileName, double newKdiffuse, double newKambient,
									   const Color &newSpecColor, int newPhongTerm) :
										Kdiffuse(newKdiffuse), Kambient(newKambient),
										specColor(newSpecColor), phongTerm(newPhongTerm) {
	imageFile = new Image(newFileName);
}

void PhongImageMaterial::shade(Color &result, const RenderContext &context,
								const Ray &ray, const HitRecord &hit,
								int depth, const Color &attenuation) const
{
	// P = O + tV
	Point hitPosition = ray.getOrigin() + (ray.getDirection() * hit.getT());
	
	// Call primitive to get normal - should already be normalized
	Vector normal;
	hit.getPrimitive()->normal(normal, hitPosition, hit, context, ray);
	
	//costheta = N dot V
	double cosTheta = dot(normal, ray.getDirection());
	
	if (cosTheta > 0)
		normal = -normal;
	else
		cosTheta = -cosTheta;
	
	// color light = scene.ambient * Ka
	Scene *myScene = context.scene;
	Color light = myScene->getAmbient() * Kambient;
	
	const Object *world = myScene->object;
	
	Color currentSpecColor(0, 0, 0);
	
	// loop through all the lights
	for (std::vector<Light*>::iterator iterator = myScene->lightList.begin();
		 iterator < myScene->lightList.end(); iterator++)
	{
		// get C.L (light color) and L (vector to light, normalized in light class)
		Color currentLightColor;
		double distance;
		Vector Lnormal;
		distance =(**iterator).getLight(currentLightColor, Lnormal, context, hitPosition);
		
		// cosphi = N dot Ln
		double cosPhi = dot(normal, Lnormal);
		
		if (cosPhi > 0.0)
		{
			HitRecord shadowHit(distance);
			
			Ray shadowRay(hitPosition, Lnormal);
			world->intersect(shadowHit, context, shadowRay);
			
			if (!shadowHit.getPrimitive()) {
				
				light += currentLightColor * Kdiffuse * cosPhi;
				
				Vector H = Lnormal - ray.getDirection();
				H.normalize();
				
				double cosAlpha = dot(H, normal);
				if (cosAlpha > 0)
				{
					double p = pow(cosAlpha, phongTerm);
					currentSpecColor += currentLightColor * p;
				}
			}
			
		}
	}
	
	Vector textCoord;
	
	hit.getPrimitive()->getTextCoord(textCoord, context, ray, hit);
	
	Color interpColor = imageFile->interpolate(textCoord.x, textCoord.y);
			
	result = light * interpColor + currentSpecColor * specColor;
}
