/*
 *  Ring.cpp
 *  Assignment3
 *
 *  Created by Zach Gildersleeve on 2/4/07.
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 *
 */

#include "Ring.h"

Ring::Ring(Material *newMaterial, const Point &newCenter, const Vector &newNormal, double newInnerRadius, double newOuterRadius)
: Primitive(newMaterial), center(newCenter), normalVector(newNormal), innerRadius(newInnerRadius), outerRadius(newOuterRadius) {
	normalVector.normalize();
	d = dot(normalVector, center);
	innerRadiusSquared = innerRadius * innerRadius;
	outerRadiusSquared = outerRadius * outerRadius;
} 

bool Ring::intersect(HitRecord &hit, const RenderContext &context, const Ray &ray) const {
	double denom = dot(normalVector, ray.getDirection());
	
	if (denom < 1.0e-7 && denom > -1.0e-7)
		return false;
	else
	{
		// numer = d + plane.normal dot ray.origin
		double numer = dot(normalVector, ray.getOrigin());
		numer = d - numer;
		double t = (numer/denom);
		//		double t = 30.0;
		
		// P = O + tV
		Point hitPoint = ray.getOrigin() + ray.getDirection() * t;
		// D = P - C
		Vector hitRadius = hitPoint - center;
		double hitRadiusLength = dot(hitRadius, hitRadius);
		if (hitRadiusLength < outerRadiusSquared && hitRadiusLength > innerRadiusSquared)
		{
			hit.hit(t, this, material);
			return true;
		}
		return false;
	}
}


void Ring::normal(Vector &normal, const Point &hitPoint, const HitRecord &hit, const RenderContext &context, const Ray &ray) const {
	normal = normalVector;
}

void Ring::getBounds(BoundingBox& bbox) const {
	double radiusSqrt = sqrt(outerRadiusSquared);
	
//	Vector diagonal = Vector(radiusSqrt * sqrt(normal.y * normal.y + normal.z * normal.z),
//					   radiusSqrt * sqrt(normal.x * normal.x + normal.z * normal.z),
//					   radiusSqrt * sqrt(normal.x * normal.x + normal.y * normal.y));
//	bbox.build(center + outerRadius);
//	bbox.build(center - outerRadius);
}
