/*
 *  Scene.cpp
 *  Assignment2
 *
 *  Created by Zach Gildersleeve on 1/28/07.
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 *
 */

#include "Scene.h"
#include <iostream>
#include <float.h>



Scene::Scene() {
	finalColor = Color(0.0, 0.0, 0.0);
}

Scene::~Scene() {
	delete background;
	delete camera;
	delete object;
	delete image;
	lightList.erase(lightList.begin(), lightList.end());
	//delete lightList;
}

void Scene::preprocess() {
	background->preprocess();
	double ratio = image->getAspectRatio();
	camera->preprocess(ratio);
	object->preprocess();
	
	//std::vector<Object *>::const_iterator i;
	
	for (int i = 0; i < lightList.size(); i++)
	{
		Light *currentLight = lightList[i];
		currentLight->preprocess();
	}
	
	xRes = image->getWidth();
	yRes = image->getHeight();
}

void Scene::render() {

	RenderContext context(this);
	Color result;
	Color initialColorAttenuation(1.0, 1.0, 1.0);

	double xMult = (xRes / 2.0) - 0.5;
	double yMult = (yRes / 2.0) - 0.5;
	double xInv = 2.0 / xRes;
	double yInv = 2.0 / yRes;
	double temp;

	for(int i=0;i<yRes;i++){
	
	if (i % 16 == 0)
		std::cout << "\tRendering Line" << i << "\n";

		double y = (i - yMult) * yInv; 
				
		for(int j=0;j<xRes;j++){ 

			double x = (j - xMult) * xInv;
			
			double *weight;
			double *xs, *ys;
			int subDivisions = sampler->numberSamples;
			int support = filter->support;
			int numberSamples = support * subDivisions;	
			int numberSamplesSquared = numberSamples * numberSamples;
			
			weight = new double[numberSamplesSquared];
			xs = new double[numberSamplesSquared];
			ys = new double[numberSamplesSquared];
			
			sampler->getSamples(x, y, support, xs, ys);
			
//			printf("samples from sampler: ");
//			for(int g = 0; g < numberSamplesSquared; g++)
//				printf("%f, %f ", xs[g], ys[g]);
//				printf("\n");
			
			filter->getWeights(weight, numberSamplesSquared, xs, ys);
			
//			printf("weights from filter: ");
//			for(int g = 0; g < numberSamplesSquared; g++)
//				printf("%f ", weight[g]);
//			printf("\n");
			
			int weightCounter;
			double totalWeight = 0.0;
			temp = 0;

			for (weightCounter = 0; weightCounter < numberSamplesSquared; weightCounter++)
				totalWeight += weight[weightCounter];

			Ray ray;

			Color tempColor = Color(0.0, 0.0, 0.0);	
//			std::cout << "making ray " << i << " " << j << "\n";

			for (weightCounter = 0; weightCounter < numberSamplesSquared; weightCounter++)
			{
//			std::cout << "making ray " << x + xs[weightCounter] * xInv << " " << y + ys[weightCounter] * yInv << "\n";
//			std::cout << "x= " << x << " xs[weightCounter]= " << xs[weightCounter] << "xInv= " << xInv << "xs[]*xInv= " <<xs[weightCounter] * xInv << " ";
//			std::cout << "y= " << y << " ys[weightCounter]= " << ys[weightCounter] << "yInv= " << yInv << "ys[]*yInv= " <<ys[weightCounter] * yInv << "\n";
				
			HitRecord hit(FLT_MAX);	
				camera->makeRay(ray, context, x + xs[weightCounter] * xInv, y + ys[weightCounter] * yInv);
				object->intersect(hit, context, ray);
				
				if(!hit.getPrimitive())
				{
					background->getBackgroundColor(result, context, ray);
				}
				else
				{
					hit.getMaterial()->shade(result, context, ray, hit, 0, initialColorAttenuation);			
				}
				tempColor += result * weight[weightCounter];
//				printf("result %f %f %f\n", result.r, result.g, result.b);
			}
//			printf("totalweight= %f\n", totalWeight);
			tempColor = tempColor * (1.0 / totalWeight);
//			printf("finalColor = %f %f %f\n", tempColor.r, tempColor.g, tempColor.b);	
			image->set(j, i, tempColor); 
			
			delete weight;
			delete xs;
			delete ys;
		} 
	} 
}

void Scene::render(int newX, int newY) {
	
	RenderContext context(this);
	Color result; 
	
	double xMult = (xRes / 2.0) + 0.5;
	double yMult = (yRes / 2.0) + 0.5;
	double xInv = 2.0 / xRes;
	double yInv = 2.0 / yRes;
			
	double y = (newY - yMult) * yInv; 
	double x = (newX - xMult) * xInv; 
			
	Ray ray;
	HitRecord hit(FLT_MAX);
	Color initialColorAttenuation(1.0, 1.0, 1.0);
	
	camera->makeRay(ray, context, x, y);
	object->intersect(hit, context, ray);
	
	if(!hit.getPrimitive())
	{
		background->getBackgroundColor(result, context, ray);
	}
	else
	{
		hit.getMaterial()->shade(result, context, ray, hit, 0, initialColorAttenuation);			
	}
	
	image->setPixel(newX, newY, result);
}

double Scene::traceRay(Color &result, const RenderContext &context, const Ray &ray,
					 int depth, const Color &attenuatedColor) const {
	
	if (depth > maxRayDepth || attenuatedColor.getMax() < minAttenuation)
	{
//		printf("reflected ray reached max Ray Depth\n");
		result = finalColor;
		return 0;
	}
	else {
		HitRecord hit(FLT_MAX);
//		printf("tracing reflected ray\n");
		object->intersect(hit, context, ray);
		
		if(hit.getPrimitive())
		{
			hit.getMaterial()->shade(result, context, ray, hit, depth, attenuatedColor);
			return hit.getT();	

		}
		else
		{
			background->getBackgroundColor(result, context, ray);
			return DBL_MAX;
		}
	}
}
	
double Scene::traceRay(Color &result, const RenderContext &context, const Ray &ray,
					int depth, const Color &attenuatedColor, const Object *insideObject) const {
						 
	 if (depth > maxRayDepth || attenuatedColor.getMax() < minAttenuation)
	 {
		 result = finalColor;
		 return 0;
	 }
	 else {
		 HitRecord hit(FLT_MAX);	
		 insideObject->intersect(hit, context, ray);
		 
		 if(hit.getPrimitive())
		 {
			 hit.getMaterial()->shade(result, context, ray, hit, depth, attenuatedColor);
			 return hit.getT();	
		 }
		 else
		 {
			 background->getBackgroundColor(result, context, ray);
			 return DBL_MAX;	
		 }
	 }
}

