/*
 *  SmoothTriangle.cpp
 *  Assignment6
 *
 *  Created by Zach Gildersleeve on 3/6/07.
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 *
 */

#include "SmoothTriangle.h"

SmoothTriangle::SmoothTriangle(Material *newMaterial, const Point &newP1, const Point &newP2, const Point &newP3) 
: Primitive(newMaterial), p1(newP1), p2(newP2), p3(newP3) {
	e1 = p1 - p3;
	e2 = p2 - p3;
	normalVector = cross(e1, e2);
	normalVector.normalize();
	bbox.build(p1);
	bbox.build(p2);
	bbox.build(p3);
}

bool SmoothTriangle::intersect(HitRecord &hit, const RenderContext &context, const Ray &ray) const {
	superCounter++;
	Vector r1 = cross(ray.getDirection(), e2);
	double denom = dot(e1, r1);
	if (fabs(denom) < 1e-8)
		return false;
	
	double invDenom = 1.0 / denom;
	
	Vector s = ray.getOrigin() - p3;
	
	double b1 = dot(s, r1) * invDenom;
	if (b1 < 0.0 || b1 > 1.0)
		return false;
	Vector r2 = cross(s, e1);
	double b2 = dot(ray.getDirection(), r2) * invDenom;
	
	if (b2 < 0.0 || b1 + b2 > 1.0)
		return false;
	
	double t = dot(e2, r2) * invDenom;
	
	hit.hit(t, this, material);
	
	normalData *nd = hit.getScratchpad<normalData>();
	nd->b1 = b1;
	nd->b2 = b2;
	
	return true;
	
}

void SmoothTriangle::normal(Vector &normal, const Point &hitPoint, const HitRecord &hit, const RenderContext &context, const Ray &ray) const {
	const normalData *nd = hit.getScratchpad<normalData>();
//	normal = Vector(p1 * nd->b1 + nd->b2 * p2 + (1.0 - nd->b1 - nd->b2) * p3);
	normal.normalize();
}

void SmoothTriangle::getBounds(BoundingBox& bbox) const {
	bbox.build(p1);
	bbox.build(p2);
	bbox.build(p3);
}
