#include "Scene.h"
#include "ConstantBackground.h"
#include "Group.h"
#include "LambertMaterial.h"
#include "PointLight.h"
#include "DirectionalLight.h"
#include "PinholeCamera.h"
#include "OrthographicCamera.h"
#include "Plane.h"
#include <stdlib.h>

static int assignment = 2;

Scene* make_scene()
{
  Scene* scene = new Scene();
  scene->setBackground(new ConstantBackground(Color(.5, .8, .9)));
  Group* world = new Group();
  world->addObject(new Sphere(new LambertMaterial(Color(1,1,1), .5, .5),
                              Point(25,25,120), 10.0));
  world->addObject(new Sphere(new LambertMaterial(Color(.1, .3, .9), .7, .3),
                              Point(1.5, 3.5, 4), 2.4));
  world->addObject(new Sphere(new LambertMaterial(Color(1, .2, .2), .7, .3),
                              Point(-0.5, -1.5, 2), 1.8));
  world->addObject(new Sphere(new LambertMaterial(Color(1,.9,.1), .6, .4),
                              Point(0.5, 1.0, 1), 1.0));

  world->addObject(new Plane(new LambertMaterial(Color(.4, .4, .7), .4, .6),
                          Vector(0,0,1), Point(0,0,-0.1)));
  scene->setObject(world);


  scene->setAmbient(Color(.6, .6, .6));
  scene->setLight(new PointLight(Point(-30, -20, 80), Color(.7,.9,.9)));
  scene->setLight(new PointLight(Point(-20, -50, 40), Color(.6,.1,.1)));

  scene->setCamera(new PinholeCamera(Point(-24, -2, 5.2),
                                     Point(1,1, 2.4),
                                     Vector(0, 0, 1),
                                     22));
//  scene->setCamera(new OrthographicCamera(Point(0.7, 0.3, 0.9), Vector(0, -5, 0), Vector(0, 0, 5)));
  
  return scene;
}

Scene* make_scene2()
{
	Scene* scene = new Scene();
	scene->setBackground(new ConstantBackground(Color(.5, .8, .9)));
	Group* world = new Group();
	world->addObject(new Sphere(new LambertMaterial(Color(1,1,1), .5, .5),
								Point(25,25,120), 10.0));
							
	world->addObject(new Sphere(new LambertMaterial(Color(.77,.45,.078), .5, .4),
								Point(0, -0.8, 1.0), 1.0));
								
																						
	world->addObject(new Sphere(new LambertMaterial(Color(.467, .082, .051), .7, .3),
								Point(0, 1.5, 0.5), 0.5));
	world->addObject(new Sphere(new LambertMaterial(Color(.467, .082, .051), .7, .3),
								Point(1.5, 1.0, 0.5), 0.5));
	world->addObject(new Sphere(new LambertMaterial(Color(.467, .082, .051), .7, .3),
								Point(-0.5, 0.7, 0.5), 0.5));
								
	world->addObject(new Sphere(new LambertMaterial(Color(.306, .702, .043), .7, .3),
								Point(0, 2.3, 0.25), 0.25));
	world->addObject(new Sphere(new LambertMaterial(Color(.306, .702, .043), .7, .3),
								Point(-0.5, 0.0, 0.25), 0.25));
								
	world->addObject(new Sphere(new LambertMaterial(Color(.235, .333, .063), .5, .7),
								Point(3, 3.5, 1.5), 1.5));
	world->addObject(new Sphere(new LambertMaterial(Color(.235, .333, .063), .5, .7),
								Point(5, -5, 4.0), 4.0));
				
	LambertMaterial *purple = new LambertMaterial(Color(.286, .024, .373), .9, .3);
			int reducer = 0;
	for(int g = 1; g < 6; g++)
	{

		double hstep = g * 0.1;
		int multiplier = 0;

		if(hstep > .15 && hstep < .4)
			multiplier = 1;
		if(hstep > .25)
			reducer++;

	for (int i = 0; i < 4 + multiplier; i++)
	{
		double r1 = (rand() % 10) * 0.01;
		double step = 0.1 * i;
		world->addObject(new Sphere(purple, Point(-1.0 - step + r1, 1.3 + step + r1, hstep), 0.1));			
	}
	for (int h = 6 + multiplier; h - reducer > 0; h--)
	{
		double step2 = h * 0.2;
		for (int i = 0; i < h; i++)
		{	
			double r1 = (rand() % 10) * 0.01;
			double step = 0.1 * i;
			world->addObject(new Sphere(purple, Point(-1.0 - step + r1, 0.0 + step2 + step + r1, hstep + r1), 0.1));			
		}
	}
																																	
	}

	
	world->addObject(new Plane(new LambertMaterial(Color(.306, .2, .11), .9, .6),
							   Vector(0,0,1), Point(0,0,-0.0)));
	scene->setObject(world);
	
	
	scene->setAmbient(Color(.8, .8, .8));
	scene->setLight(new PointLight(Point(-30, -20, 80), Color(.8,.8,.4)));
	scene->setLight(new PointLight(Point(-20, -50, 40), Color(.4,.4,.8)));
	
	scene->setCamera(new PinholeCamera(Point(-10, -2.8, 4),
									   Point(1,1, 1.1),
									   Vector(0, 0, 1),
									   22));
	
	return scene;
}
