#include "Scene.h"
#include "ConstantBackground.h"
#include "Group.h"
#include "LambertMaterial.h"
#include "PointLight.h"
#include "DirectionalLight.h"
#include "PinholeCamera.h"
#include "OrthographicCamera.h"
#include "Plane.h"
#include "Disc.h"
#include "Ring.h"
#include "Triangle.h"
#include "Box.h"
#include "Cylinder.h"
#include "HeightField.h"
#include "PhongMaterial.h"
#include "MetalMaterial.h"
#include "DielectricMaterial.h"
#include "GriddedGroup.h"
#include "PhongMarbleMaterial.h"
//#include "SmoothTriangle.h"
#include "WoodMaterial.h"
#include "TileMaterial.h"
#include "Filter.h"
#include "Sampler.h"
#include "PhongImageMaterial.h"
#include "SpherePolar.h"



static int assignment = 6;

#include <fstream>
#include <iostream>
#include <vector>
using std::cerr;
using std::ifstream;
using std::vector;

static void error()
{
  cerr << "Error reading triangle mesh\n";
  exit(1);
}

Scene* make_scene()
{
  Scene* scene = new Scene();
  scene->setBackground(new ConstantBackground(Color(0.0, 0.0, 0.0)));
  Group* world = new Group();
  GriddedGroup* bunny = new GriddedGroup(10);
  world->addObject(bunny);
  
  Material* marbleBlue = new PhongMarbleMaterial(Color(0.0, 0.0, 0.0), // Color 1
												 Color(1.0, 1.0, 1.0), // Color 2
												 2, // scale
												 10, // octaves
												 20.0, // turbulence scale
												 1.0, // frequency scale
												 2.0, // lacunarity
												 0.6, // gain
												 0.9, 0.9, Color(1,1,1), 120);
  Material* tile = new TileMaterial(Point(0,0,0), Vector(0.04,0.0,0), Vector(0.0,0.0,0.04),
									new PhongMaterial(Color(1.0, 1.0, 1.0), 0.5, 0.5, Color(1,1,1), 60),
									marbleBlue, 0.05);

  Material* pedestalmatl = new MetalMaterial(Color(0.96, 0.89, 0.37), 200);
  world->addObject(new Box(pedestalmatl,
                           Point(-0.08, 0.024, -0.08), Point(0.08, 0.033, 0.08)));
  Material* yellow = new PhongMaterial(Color(0.9, 0.9, 0), 0.6, 0.4, Color(1,0,0), 30);	
  world->addObject(new Box(tile,
                           Point(-0.15, 0, -0.15), Point(0.15, 0.024, 0.15)));

  Material * wood = new WoodMaterial();	
  Material* bunnymatl = new PhongMaterial(Color(0.6, 0.4, .3), 0.6, 0.4,
                                          Color(1,1,1), 80);
										  
  Material* image = new PhongImageMaterial("/Users/zachgildersleeve/Documents/SocS2007/cs6620-RayTracing/Assignment7/land_ocean_ice_cloud_8192.ppm", 0.6, 0.4, Color(1,1,1), 80);
  world->addObject(new SpherePolar(image,
                                   Point(.5, 0, -1.5), 1.0,
                                   Vector(0,-.4,1), Vector(-1,-.2,0)));
										  
										  
Vector size = Vector(0.001, 0.001, 0.001);
  ifstream in("/Users/zachgildersleeve/Documents/SocS2007/cs6620-RayTracing/Assignment6/bunny.tm");
  if(!in)
    error();
  bool have_normals, have_something_else;
  in >> have_normals >> have_something_else;
  int numverts;
  in >> numverts;
  if(!in)
    error();
  cerr << "reading " << numverts << " vertices...";
  vector<Point> verts(numverts);
  for(int i=0;i<numverts;i++){
    double x,y,z;
    in >> x >> y >> z;
    if(!in)
      error();
    verts[i] = Point(x,y,z);
  }
  int numtris;
  in >> numtris;
  cerr << " reading " << numtris << " triangles...";
  for(int i=0;i<numtris;i++){
    int i1, i2, i3;
    in >> i1 >> i2 >> i3;
    if(!in)
      error();
    bunny->addObject(new Triangle(wood, verts[i1], verts[i2], verts[i3]));
  }
  cerr << " done\n";
		    
  scene->setObject(world);
  
  int subDivisions = 3;
  int support = 3;
  scene->setSampler(new JitteredSampler(subDivisions));
  scene->setFilter(new GaussianFilter(support));
  

  scene->setAmbient(Color(.4, .4, .4));
  scene->addLight(new PointLight(Point(20, 70, 100), Color(.9,.9,.9)));
  scene->addLight(new PointLight(Point(-40, -70, 50), Color(.3,.1,.1)));

  scene->setCamera(new PinholeCamera(Point(-.7,.5,.7),
                                     Point(-0.04, .10, -0.00),
                                     Vector(0, 1, 0),
                                     12));
  scene->setMaxRayDepth(25);
  scene->setMinAttenuation(.01);

  return scene;
}



//Scene* make_scene()
//{
//  Scene* scene = new Scene();
//  scene->setBackground(new ConstantBackground(Color(0.0, 0.0, 0.0)));
//  Group* world = new Group();
//  GriddedGroup* bunny = new GriddedGroup(10);
//  
//  world->addObject(bunny);
//
//  Material* pedestalmatl1 = new LambertMaterial(Color(0.9, 0.1, 0.3), 0.7, 0.3);
//  
//  
//  Material* red = new PhongMaterial(Color(1.0, 0.18, .18), 0.6, 0.4, Color(1,8,8), 30);
//  Material* brown = new PhongMaterial(Color(0.6, 0.4, .3), 0.6, 0.4, Color(1,1,1), 80);									  
//  Material* blue = new PhongMaterial(Color(0.05, 0.1, .03), 0.6, 0.4, Color(1,1,1), 180);									  
//  Material* green = new PhongMaterial(Color(0.47, 0.75, .61), 0.6, 0.4, Color(0,0,1), 100);									  
//  Material* yellow = new PhongMaterial(Color(0.9, 0.9, 0), 0.6, 0.4, Color(1,0,0), 30);	
//  Material* cream = new PhongMaterial(Color(0.98, 0.98, 0.76), 0.6, 0.4, Color(1,0,0), 30);								  
//  Material* goldMetal = new MetalMaterial(Color(0.96, 0.89, 0.37), 200);
//  Material* silverMetal = new MetalMaterial(Color(0.96, 0.96, 0.96), 200);
//  Material* glass = new DielectricMaterial(1.3, 100, Color(0.5, 0.0, 0.5));
//
//
//										  
//  ifstream in("/Users/zachgildersleeve/Documents/SocS2007/cs6620-RayTracing/Assignment6/bunny.tm");
//  if(!in)
//    error();
//  bool have_normals, have_something_else;
//  in >> have_normals >> have_something_else;
//  int numverts;
//  in >> numverts;
//  if(!in)
//    error();
//  cerr << "reading " << numverts << " vertices...";
//  vector<Point> vertsC(numverts);
//  vector<Point> vertsLC(numverts);
//  vector<Point> vertsRC(numverts);
//  vector<Point> vertsB(numverts);
//  vector<Point> vertsLB(numverts);
//  vector<Point> vertsRB(numverts);
//  vector<Point> vertsLU(numverts);
//  vector<Point> vertsLUs(numverts);
//  vector<Point> vertsU(numverts);
//  vector<Point> vertsRU(numverts);
//
//  for(int i=0;i<numverts;i++){
//    double x,y,z;
//    in >> x >> y >> z;
//    if(!in)
//      error();
//	vertsC[i] = Point(x,y-=0.1,z);
//    vertsLC[i] = Point(x-=0.21,y,z);
//	vertsRC[i] = Point(x+=0.47,y,z);
//	vertsB[i] = Point(x-=0.23,y-=0.22,z);
//	vertsLB[i] = Point(x-=0.21,y,z);
//	vertsRB[i] = Point(x+=0.47,y,z);
//	vertsRU[i] = Point(x,y+=0.47,z);
//	vertsU[i] = Point(x-=0.27,y,z);
//	vertsLU[i] = Point(x-=0.22,y,z);
////	vertsLUs[i] = Point(x*=0.5,y*=0.5,z*=0.5);
//
//  }
//  int numtris;
//  in >> numtris;
//  cerr << " reading " << numtris << " triangles...";
//  for(int i=0;i<numtris;i++){
//    int i1, i2, i3;
//    in >> i1 >> i2 >> i3;
//    if(!in)
//      error();
//	  bunny->addObject(new Triangle(red, vertsC[i1], vertsC[i2], vertsC[i3]));
//	  bunny->addObject(new Triangle(brown, vertsLC[i1], vertsLC[i2], vertsLC[i3]));
//	  
//	  if (i % 15 == 0)
//		  bunny->addObject(new Sphere(red, vertsRC[i1], 0.001));  
//	  else
//		  bunny->addObject(new Triangle(blue, vertsRC[i1], vertsRC[i2], vertsRC[i3]));
//	  
//	  
//	  bunny->addObject(new Triangle(green, vertsB[i1], vertsB[i2], vertsB[i3]));
//	  bunny->addObject(new Triangle(goldMetal, vertsLB[i1], vertsLB[i2], vertsLB[i3]));
//	  bunny->addObject(new Triangle(yellow, vertsRB[i1], vertsRB[i2], vertsRB[i3]));
//	  bunny->addObject(new Triangle(cream, vertsLU[i1], vertsLU[i2], vertsLU[i3]));
////	bunny->addObject(new Triangle(cream, vertsLUs[i1], vertsLUs[i2], vertsLUs[i3]));
//
//	  bunny->addObject(new Triangle(silverMetal, vertsU[i1], vertsU[i2], vertsU[i3]));
//	  bunny->addObject(new Triangle(glass, vertsRU[i1], vertsRU[i2], vertsRU[i3]));
//  }
//  cerr << " done\n";
//
//
//
//
//  world->addObject(new Box(yellow,	Point(0.135, -0.135, -0.5), Point(0.405, 0.135, -0.4))); // RC
//  world->addObject(new Box(cream,	Point(-0.135, -0.135, -0.5), Point(0.135, 0.135, -0.4))); // C
//  world->addObject(new Box(red,		Point(-0.405, -0.135, -0.5), Point(-0.135, 0.135, -0.4))); // LC
//  
//  world->addObject(new Box(yellow,	Point(-0.405, -0.405, -0.5), Point(-0.135, -0.135, -0.4))); // LB
//  world->addObject(new Box(green,	Point(0.135, -0.405, -0.5), Point(0.405, -0.135, -0.4))); // RB
//  world->addObject(new Sphere(silverMetal, Point(0.0 , -0.2, -3.0), 1.0));
//
//  
//  world->addObject(new Box(yellow,	Point(-0.405, 0.135, -0.5), Point(-0.135, 0.405, -0.4))); // LU
//  bunny->addObject(new Sphere(silverMetal, Point(-0.35 , 0.35, -0.4), 0.01));
//  bunny->addObject(new Sphere(silverMetal, Point(-0.35 , 0.2, -0.4), 0.01));
//  bunny->addObject(new Sphere(silverMetal, Point(-0.2 , 0.35, -0.4), 0.01));
//  bunny->addObject(new Sphere(silverMetal, Point(-0.2 , 0.2, -0.4), 0.01));
//
//  bunny->addObject(new Sphere(silverMetal, Point(0.2 , 0.2, -0.4), 0.01));
// 
//  
//  
//  world->addObject(new Box(goldMetal,	Point(-0.135, 0.135, -0.5), Point(0.135, 0.405, -0.4))); // U
//  world->addObject(new Sphere(brown, Point(0.5 , 0.5, -1.0), 0.5));
//
//		    
//  scene->setObject(world);
//
//  scene->setAmbient(Color(.4, .4, .4));
//  scene->addLight(new PointLight(Point(20, 70, 100), Color(.9,.9,.9)));
//  scene->addLight(new PointLight(Point(-40, -70, 50), Color(.3,.1,.1)));
//
//  scene->setCamera(new PinholeCamera(Point(0,.00,3.5),
//                                     Point(0.00, 0.00, 0.00),
//                                     Vector(0, 1, 0),
//                                     12));
//  scene->setMaxRayDepth(25);
//  scene->setMinAttenuation(.01);
//
//  return scene;
//}
