#include "Scene.h"
#include "ConstantBackground.h"
#include "Group.h"
#include "LambertMaterial.h"
#include "PointLight.h"
#include "DirectionalLight.h"
#include "PinholeCamera.h"
#include "OrthographicCamera.h"
#include "Plane.h"
#include "Disc.h"
#include "Ring.h"
#include "Triangle.h"
#include "Box.h"
#include "Cylinder.h"
#include "HeightField.h"
#include "PhongMaterial.h"
#include "MetalMaterial.h"
#include "DielectricMaterial.h"
#include "Filter.h"
#include "Sampler.h"

static int assignment = 8;

Scene* make_scene()
{
	Scene* scene = new Scene();
	scene->setBackground(new ConstantBackground(Color(0.5, 0.8, 0.9)));
	Group* world = new Group();
	Material* groundmatl = new LambertMaterial(Color(0.65, 0.75, 0.85), .6, .4);
//	world->addObject(new Plane(groundmatl,
//							   Vector(0,0,1), Point(0,0,1.5)));
	world->addObject(new Triangle(groundmatl, Point(0,0,1.5), Point(1.5,0,1.5), Point(0,0,0)));
	
	for(int i=0;i<4;i++){
		double eta = 1 + i*0.5 + .05;
		Material* transp_matl = new DielectricMaterial(eta, 100);
		Point corner(i*1.3 - 4, -3, 1.51);
		Vector size(0.20, 2.5, 1.4);
		world->addObject(new Box(transp_matl,
								 corner, corner+size));
	}
	
	Material* ringmatl = new DielectricMaterial(.9, 100, Color(0.0, 1.0, 1.0));
	for(int i=0;i<15;i++){
		double r = .30;
		Point center(-9, 0, 1.6+r);
		Vector offset(2.35*r, 0, 0);
//		world->addObject(new Ring(ringmatl,
//								  center+offset*i, Vector(0.2, -1, -0.2), r*0.5, r));
	}
	Material* ball_matl = new MetalMaterial(Color(0.8, 0.8, 0.8), 100);
	world->addObject(new Sphere(ball_matl, Point(-7, 3.5, 2.4), 1.2));
	
	Material* hfmatl = new DielectricMaterial(5.0, 100, Color(0.7, 0.1, .3));
//	world->addObject(new HeightField(hfmatl,
//									 "/Users/zachgildersleeve/Documents/SocS2007/cs6620-RayTracing/Assignment4/hf1.hf",
//									 Point(-4.5, 2.0, 2), Point(-1.5, 5.0 , 4)));
	scene->setObject(world);
	
	int subDivisions  = 1;
	int support = 1;
	
	scene->setSampler(new UniformSampler(subDivisions));
	scene->setFilter(new BoxFilter(support));
	
	scene->setAmbient(Color(.4, .4, .4));
	scene->addLight(new PointLight(Point(20, -20, 100), Color(.9,.9,.9)));
	scene->addLight(new PointLight(Point(-40, -30, 50), Color(.3,.1,.1)));
	
	scene->setCamera(new PinholeCamera(Point(8, -18, 7.5),
									   Point(-4.7, 2.5, 1.5),
									   Vector(0, 0, 1),
									   15));
	scene->setMaxRayDepth(25);
	scene->setMinAttenuation(.01);
	
	return scene;
}

//Scene* make_scene()
//{
//	Scene* scene = new Scene();
//	scene->setBackground(new ConstantBackground(Color(0.1, 0.2, 0.3)));
//	Group* world = new Group();
//	
//	Material* groundmatl = new LambertMaterial(Color(0.65, 0.75, 0.85), .6, .4);
//	Material* ball_matl = new MetalMaterial(Color(0.8, 0.8, 0.8), 100);
//	Material* hfmatl = new PhongMaterial(Color(0.7, 0.1, .3), 0.6, 0.4, Color(1,1,1), 30);
//	Material* whitePhong = new PhongMaterial(Color(0.9, 0.9, 0.9), 1.0, 1.0, Color(1,1,1), 30);
//	Material* whiteTransparent = new DielectricMaterial(1.5, 100);
//	Material* clearTransparent = new DielectricMaterial(3.5, 100);
//	Material* greenTransparent = new DielectricMaterial(1.5, 100, Color(39.0/255.0, 95.0/255.0, 11.0/255.0));
//	Material* blueTransparent = new DielectricMaterial(4.5, 50, Color(14.0/255.0, 40.0/255.0, 114.0/255.0));
//	Material* blackTransparent = new DielectricMaterial(1.5, 100, Color(0.0, 0.0, 0.0));
//	
//	world->addObject(new Disc(ball_matl, Point(0, 3, 0), Vector(0, .5, .5), 3.0));
//	world->addObject(new Disc(ball_matl, Point(0, 1.8, 0), Vector(0, 1, 0), 1.0));
//	world->addObject(new Disc(ball_matl, Point(0, 0, -11), Vector(0, 0, 1), 10.0));
//
//	world->addObject(new Sphere(greenTransparent, Point(0, 2, .4), 0.2));
//	world->addObject(new Sphere(greenTransparent, Point(0, 2, -.4), 0.2));
//	world->addObject(new Sphere(greenTransparent, Point(0.4, 2, 0), 0.2));
//	world->addObject(new Sphere(greenTransparent, Point(-0.4, 2, 0), 0.2));
//	
//	world->addObject(new Sphere(whiteTransparent, Point(-0.6, 2, 0.6), 0.2));
//	world->addObject(new Sphere(whiteTransparent, Point(-0.6, 2, -0.6), 0.2));
//	world->addObject(new Sphere(whiteTransparent, Point(0.6, 2, 0.6), 0.2));
//	world->addObject(new Sphere(whiteTransparent, Point(0.6, 2, -0.6), 0.2));
//	
//	world->addObject(new Sphere(blueTransparent, Point(0.3, 2, -0.8), 0.2));
//	world->addObject(new Sphere(blueTransparent, Point(-0.3, 2, -0.8), 0.2));
//	world->addObject(new Sphere(blueTransparent, Point(0.3, 2, 0.8), 0.2));
//	world->addObject(new Sphere(blueTransparent, Point(-0.3, 2, 0.8), 0.2));
//
//	world->addObject(new Sphere(blueTransparent, Point(0.8, 2, -0.3), 0.2));
//	world->addObject(new Sphere(blueTransparent, Point(0.8, 2, 0.3), 0.2));
//	world->addObject(new Sphere(blueTransparent, Point(-0.8, 2, 0.3), 0.2));
//	world->addObject(new Sphere(blueTransparent, Point(-0.8, 2, -0.3), 0.2));
//	
//	Point corner(-1, 0, 3.0);
//	Vector size(0.20, 2.5, 1.4);
//	
//	world->addObject(new Sphere(hfmatl, Point(0.0, 2.0, 0.0), 0.2));
//
//	scene->setObject(world);
//	
//	scene->setAmbient(Color(.8, .8, .8));
//	scene->addLight(new PointLight(Point(10, 40, -10), Color(.9,.9,.9)));
//	scene->addLight(new PointLight(Point(0, 20, 0), Color(.9,.9,.9)));
//	scene->addLight(new PointLight(Point(-40, 30, -50), Color(.3,.1,.1)));
//	scene->addLight(new PointLight(Point(-10, 40, -10), Color(.9,.9,.9)));
//	
//	scene->setCamera(new PinholeCamera(Point(0, 5.5, -10),
//									   Point(0, 2.5, 0),
//									   Vector(0, 1, 0),
//									   10));
//	scene->setMaxRayDepth(25);
//	scene->setMinAttenuation(.01);
//	
//	return scene;
//}
	
//Scene* make_scene()
//{
//	Scene* scene = new Scene();
//	scene->setBackground(new ConstantBackground(Color(0.9, 0.9, 0.9)));
//	Group* world = new Group();
//	Material* groundmatl = new LambertMaterial(Color(0.65, 0.75, 0.85), .6, .4);
//	
//	
//	double eta = 1.05;
//	
//	Material* ball_matl = new MetalMaterial(Color(0.8, 0.8, 0.8), 100);
//	
//	Material* hfmatl = new PhongMaterial(Color(0.7, 0.1, .3), 0.6, 0.4, Color(1,1,1), 30);
//	Material* brownPhong = new PhongMaterial(Color(0.31, 0.2, 0.02), .2, .2, Color(.31,.2,.02), 30);
//	
//	world->addObject(new Disc(ball_matl, Point(0, 0, 0), Vector(-1, 1, 0), 2.0));
//	world->addObject(new Disc(brownPhong, Point(.1, 0, 0), Vector(-1, 1, 0), 2.4));
//	
//	Material* greenTransparent = new DielectricMaterial(1.5, 100);//, Color(39.0/255.0, 95.0/255.0, 11.0/255.0));
//
//	Point corner(0, 0, 0);
//	double scale = 0.3;
//	Vector size(scale, scale, scale);
//    world->addObject(new Box(greenTransparent, corner, corner+size));
//	
//	for (int h = 1; h < 3; h++)
//	 {
//		 for (int i = 0; i < (h * 2 + 1); i++)
//		 {
//			 for (int j = 0; j < (h * 2 + 1); j++)
//			 {
//				 corner = Point(-(h * scale) + (i * scale), (h * scale), -(h * scale) + (j * scale));
//				 
//				 if (h == 9)
//					 world->addObject(new Box(greenTransparent, corner, corner+size));			 
//				 else if (i == 0 || j == 0 || i == h * 2 || j == h * 2)
//					 world->addObject(new Box(greenTransparent, corner, corner+size));
//			 }
//		 }
//	 }
//	
//	scene->setObject(world);
//	
//	scene->setAmbient(Color(.8, .8, .8));
//	scene->addLight(new PointLight(Point(10, 40, -10), Color(.9,.9,.9)));
//	scene->addLight(new PointLight(Point(0, 20, 0), Color(.9,.9,.9)));
//	scene->addLight(new PointLight(Point(-40, 30, -50), Color(.3,.1,.1)));
//	scene->addLight(new PointLight(Point(-10, 40, -10), Color(.9,.9,.9)));
//	
//	scene->setCamera(new PinholeCamera(Point(-20, 10, -25),
//									   Point(0, 0, 0),
//									   Vector(-1, 1, 0),
//									   10));
//	scene->setMaxRayDepth(25);
//	scene->setMinAttenuation(.01);
//	
//	return scene;
//}
	
